#include "charts/statsgeneralchartwidget.h"


StatsGeneralChartWidget::StatsGeneralChartWidget()
{
      //qDebug() << "StatsGeneralChartWidget::StatsGeneralChartWidget"  << endl;
}

StatsGeneralChartWidget::StatsGeneralChartWidget(DataProxy_SQLite *dp, QWidget *parent)
{
      //qDebug() << "StatsGeneralChartWidget::StatsGeneralChartWidget"  << endl;
}


void StatsGeneralChartWidget::prepareChart() {}
void StatsGeneralChartWidget::createUI(){}
